using data.Domain.UseCase;
using data.RemoteData.RemoteDataBase;
using data.Repository;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Logging;

var builder = WebApplication.CreateBuilder(args);

//      
builder.Services.AddDbContext<RemoteDatabaseContext>();

//    UseCase
builder.Services.AddScoped<IUserRepository, SQLUserRepositoryImpl>()
                .AddScoped<IPresenceRepository, SQLPresenceRepositoryImpl>()
                .AddScoped<IGroupRepository, SQLGroupRepositoryImpl>()
                .AddScoped<UseCaseGeneratePresence>()
                .AddScoped<GroupUseCase>();

//  
builder.Services.AddControllers();

//  Swagger  API 
builder.Services.AddEndpointsApiExplorer();
builder.Services.AddSwaggerGen();

//  
builder.Services.AddLogging(logging =>
{
    logging.ClearProviders();
    logging.AddConsole(options =>
    {
        options.LogToStandardErrorThreshold = LogLevel.Error;
    });
    logging.AddDebug();
    logging.SetMinimumLevel(LogLevel.Error);
});

var app = builder.Build();

//  Swagger UI  
if (app.Environment.IsDevelopment())
{
    app.UseSwagger();
    app.UseSwaggerUI();
}

//  middleware    
app.UseMiddleware<ErrorLoggingMiddleware>();

//  Middleware
app.UseHttpsRedirection();
app.MapControllers();

app.Run();

public class ErrorLoggingMiddleware
{
    private readonly RequestDelegate _next;
    private readonly ILogger<ErrorLoggingMiddleware> _logger;

    public ErrorLoggingMiddleware(RequestDelegate next, ILogger<ErrorLoggingMiddleware> logger)
    {
        _next = next;
        _logger = logger;
    }

    public async Task InvokeAsync(HttpContext context)
    {
        try
        {
            //     
            await _next(context);

            //       4xx  5xx
            if (context.Response.StatusCode >= 400)
            {
                var logLevel = context.Response.StatusCode >= 500 ? LogLevel.Error : LogLevel.Warning;

                _logger.Log(logLevel, "  : {StatusCode}  {Method} {Path}",
                    context.Response.StatusCode, context.Request.Method, context.Request.Path);
            }
        }
        catch (Exception ex)
        {
            //   
            _logger.LogError(ex, "    {Method} {Path}: {Message}",
                context.Request.Method, context.Request.Path, ex.Message);

            //   
            context.Response.StatusCode = 500;
            await context.Response.WriteAsync("   .");
        }
    }
}

//      
[ApiController]
[Route("api/[controller]")]
public class TestController : ControllerBase
{
    [HttpGet("bad-request")]
    public IActionResult BadRequestExample()
    {
        return BadRequest("   400.");
    }

    [HttpGet("internal-error")]
    public IActionResult InternalServerErrorExample()
    {
        throw new InvalidOperationException("   .");
    }
}
